#ifndef MODEL_H
#define MODEL_H

#include <vector>
#include <string>

class Mesh;
class Image;
class TexturePack;

class Model
{
protected:
    TexturePack *m_texPack;
    std::vector<Mesh*> m_meshes;

public:
    Model();
    
    ~Model();
    
    void setTexturePack(TexturePack *texPack);
    
    /**
     * @brief addMesh adds a mesh to the mesh list
     */
    void addMesh(Mesh *mesh) { m_meshes.push_back(mesh); }
    
    /**
     * @brief getMeshes returns the mesh array
     */
    const std::vector<Mesh*>& getMeshes() { return m_meshes; }
    
    /**
     * @brief getImage returns the image which has the specified name, or NULL if there are no images of this name.
     */
    //Image* getImage(const std::string &name);
    
    /**
     * @brief getTexture returns the texture which has the specified name, or NULL if there are no images of this name.
     */
    //Texture* getTexture(const std::string &name);
    
    /**
     * @brief uploads meshes and images to GPU memory,
     * then frees Mesh and Image memory in the RAM,
     * so it stays allocated only in GPU memory.
     */
    void initDestroy();
    
    /**
     * @brief initGL uploads image and mesh data to GPU memory
     */
    void initGL();
    
    /**
     * @brief destroyGL frees the meshes and the textures from GPU memory
     */
    void destroyGL();
    
    /**
     * @brief save saves the model on a file
     * @param filename is the name of the resulting file
     * @param texturesPath is the existing folder where the textures will be saved
     * @return true if the save succeeded
     */
    bool save(const std::string &filename, const std::string &texturesPath);
    
    /**
     * @brief load loads the model from a file
     * @param filename is the name of the target file
     * @param texturesPath is the existing folder where the textures will be gathered
     * @return the model if the loading succeeded, or NULL if it failed
     */
    static Model* load(const std::string &filename, const std::string &texturesPath);
};

#endif // MODEL_H
